from os import path, remove, stat, makedirs


class BaseFile():
    """
    Base file class that can be utilized by any
    object that represents a file. The class is
    initialized with a filepath, either relative or
    hardcoded.
    """

    def __init__(self, filepath):
        self.filepath = filepath

    def set_filepath(self, filepath):
        self.filepath = filepath

    def get_filepath(self):
        return self.filepath

    def create_filepath(self):
        """
        Creates filepaths for both dirs and files.
        First creates the directory paths and then tries
        to create the filepath if the filepath is a file.
        """
        makedirs(path.dirname(self.filepath), exist_ok=True)
        try:
            with open(self.filepath, "a") as f:
                f.write("")
        except Exception:
            pass

    def delete_file(self):
        """
        Utilizes the remove method from the os module.
        Checks if the filepath exists and that the path
        is not a dir before deleting.
        """
        if self.filepath_exists() and self.is_file():
            remove(self.filepath)
        else:
            print("The file could not be deleted because "
                  + self.filepath + " does not exist or it is a directory.")

    def get_contents_of_file(self):
        """
        Returns all the text read from the file.
        """
        data = None
        with open(self.filepath, 'r') as my_file:
            data = my_file.read()
        return data

    def append_data_to_file(self, data):
        """
        Appends text to the file. Does not delete
        the file or clear the contents.
        """
        with open(self.filepath, 'a') as f:
            f.write(data)
            f.close()

    def write_data_to_file(self, data):
        """
        Truncates the file and writes the
        data as text.
        """
        with open(self.filepath, 'w') as f:
            f.write(data)
            f.close()

    def clear_file(self):
        """
        Checks if the file exists before truncating
        the file. Performs an open, write, close which
        clears all the contents.
        """
        if self.filepath_exists():
            open(self.filepath, 'w').close()
        else:
            print("The file could not be cleared because "
                  + self.filepath + " does not exist.")

    def is_binary(self):
        """
        A simple method to determine if the file
        is not a text file. If UnicodeDecodeError
        is thrown, it is not a text file. In our
        case, we are assuming it is a binary. This
        method is called at the initialization of
        EncryptionFile, so it needs to work when
        the file does not exist, hence the
        FileNotFoundError.
        """
        try:
            with open(self.filepath, "r") as f:
                f.read()
        except UnicodeDecodeError:
            return True
        except FileNotFoundError:
            return False
        return False

    def is_empty(self, verbose_flag=False):
        """
        Returns boolean if size of file is 0.
        """
        if stat(self.filepath).st_size == 0:
            return True
        else:
            return False

    def is_dir(self, verbose_flag=False):
        """
        Utilizes the path(filepath).isdir() function
        from the os module.
        """
        return path.isdir(self.filepath)

    def is_file(self, verbose_flage=False):
        """
        Utilizes the path(filepath).isfile() function
        from the os module.
        """
        return path.isfile(self.filepath)

    def filepath_exists(self):
        """
        Returns true if the file is a dir or a file.
        """
        if path.isdir(self.filepath) or path.exists(self.filepath):
            return True
        else:
            return False

    def __str__(self):
        return self.filepath



from Crypto.PublicKey import RSA

from .BaseFile import BaseFile


class RSAFile(BaseFile):
    """
    Inherits all the functions from BaseFile.
    An abstraction of an RSA file saved as a
    pem file. Allows the user to generate the
    key in a pem file and grab the key for
    use in encryption.
    """

    def __init__(self, filepath):
        self.filepath = filepath

    def gen_key(self):
        """
        Generates the RSA key.
        """
        return RSA.generate(2048)

    def gen_pem_file(self):
        """
        Calls the gen_key function and stores the
        key into a pem file.
        """
        key = self.gen_key()
        f = open(self.filepath, 'wb')
        f.write(key.export_key('PEM'))
        f.close()

    def get_key(self):
        """
        Returns the RSA key from the file.
        """
        key = None

        if self.filepath_exists():
            with open(self.filepath, 'r') as pem_file:
                key = RSA.import_key(pem_file.read())
        else:
            print(self.filepath + " does not exist.")

        return key



from Crypto.Cipher import PKCS1_v1_5 as Cipher_PKCS1_v1_5
from Crypto.PublicKey import RSA
from os import path, strerror
from errno import ENOENT


class Encryptor():
    """
    This class utilizes the Crypto PyPi package.
    Accepts an RSA key as an argument. The
    encrypt and decrypt class use the key
    with the Cipher_PKCS1_v1_5 algorithm.
    """

    def __init__(self, rsa_filepath):
        if path.exists(rsa_filepath):
            self.pem_key = self.get_key(rsa_filepath)
        else:
            raise FileNotFoundError(
                ENOENT, strerror(ENOENT),
                rsa_filepath)

    def encrypt_data(self, data):
        cipher = Cipher_PKCS1_v1_5.new(self.pem_key)
        if cipher.can_encrypt():
            return cipher.encrypt(data.encode())
        else:
            print("ERROR: cannot encrypt with pem.")

    def decrypt_data(self, data):
        decipher = Cipher_PKCS1_v1_5.new(self.pem_key)
        return decipher.decrypt(data, None).decode()

    def get_key(self, rsa_filepath):
        with open(rsa_filepath, 'r') as pem_file:
            key = RSA.import_key(pem_file.read())
        return key



from .BaseFile import BaseFile
from .Encryptor import Encryptor


class EncryptionFile(BaseFile):
    """
    EncryptionFile class inherits from BaseFile. EncryptionFile allows you to
    encrypt and decrypt the contents of the file. Constructor requires a
    filepath to an RSA key. The RSA key is passed to the Encryptor
    class which does the encryption and decryption.
    """

    def __init__(self, filepath, rsa_filepath):
        self.filepath = filepath
        self.rsa_filepath = rsa_filepath
        self.encryptor = Encryptor(str(rsa_filepath))
        self.is_encrypted = self.is_binary()

    def decrypt(self):
        """
        Checks if the file is encrypted and exists,
        then utilizes the Encryptor class to decrypt
        the data and write the text to the file.
        """
        if self.filepath_exists() and self.is_encrypted:
            decrypted_data = self.encryptor.decrypt_data(
                self.get_bytes_from_file())
            self.write_data_to_file(decrypted_data)
            self.is_encrypted = False
        else:
            print(self.get_filepath() + " does not exist.")

    def encrypt(self):
        """
        Checks that the file is not encrypted and exists,
        then encrypts the contents through the Encryptor
        class and writes the bytes to the file.
        """
        if self.filepath_exists() and not self.is_encrypted:
            encrypted_data = self.encryptor.encrypt_data(
                self.get_contents_of_file())
            self.write_bytes_to_file(encrypted_data)
            self.is_encrypted = True
        else:
            print(self.get_filepath() + " does not exist.")

    def write_bytes_to_file(self, data):
        """
        Writes data to the file as bytes.
        """
        if self.filepath_exists():
            with open(self.filepath, 'wb') as env_file:
                env_file.write(data)
                env_file.close()
        else:
            print(self.get_filepath() + " does not exist.")

    def get_bytes_from_file(self):
        """
        Returns the contents of the encrypted file
        as bytes.
        """
        data = None
        with open(self.filepath, 'rb') as my_file:
            data = my_file.read()
        return data




from distutils.core import setup
from setuptools import find_packages, setup
from pathlib import Path

HERE = Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name = 'fileflamingo',         # How you named your package folder (MyLib)
    packages = find_packages(exclude=("tests",)),
    version = '0.0.5',      # Start with a small number and increase it with every change you make
    license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description = 'A suite of Python base classes that simplify interactions with files.',   # Give a short description about your library
    author = 'Jeff Gruenbaum',
    author_email = 'jeff.gruenbaum@gmail.com',
    long_description=README,
    long_description_content_type="text/markdown",
    url = 'https://github.com/jgrugru/file-flamingo', 
    keywords = ["file", 'file abstraction', 'file functions', 'fileclass', 'class file', 'mixin'],
    install_requires=[
        "pycryptodome==3.10.1",
    ],
    include_package_data=True,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
)

# flake8: noqa